@@ -713,6 +713,31 @@ cleanup_ma:
 static void (*tdb_robust_mutext_old_handler)(int) = SIG_ERR;
 static pid_t tdb_robust_mutex_pid = -1;
 
+static bool tdb_robust_mutex_setup_sigchild(void (*handler)(int),
+			void (**p_old_handler)(int))
+{
+#ifdef HAVE_SIGACTION
+	struct sigaction act;
+	struct sigaction oldact;
+
+	memset(&act, '\0', sizeof(act));
+
+	act.sa_handler = handler;
+#ifdef SA_RESTART
+	act.sa_flags = SA_RESTART;
+#endif
+	sigemptyset(&act.sa_mask);
+	sigaddset(&act.sa_mask, SIGCHLD);
+	sigaction(SIGCHLD, &act, &oldact);
+	if (p_old_handler) {
+		*p_old_handler = oldact.sa_handler;
+	}
+	return true;
+#else /* !HAVE_SIGACTION */
+	return false;
+#endif
+}
+
 static void tdb_robust_mutex_handler(int sig)
 {
 	if (tdb_robust_mutex_pid != -1) {
@@ -803,8 +828,10 @@ _PUBLIC_ bool tdb_runtime_check_for_robust_mutexes(void)
 		goto cleanup_ma;
 	}
 
-	tdb_robust_mutext_old_handler = signal(SIGCHLD,
-					       tdb_robust_mutex_handler);
+	if (tdb_robust_mutex_setup_sigchild(tdb_robust_mutex_handler,
+			&tdb_robust_mutext_old_handler) == false) {
+		goto cleanup_ma;
+	}
 
 	tdb_robust_mutex_pid = fork();
 	if (tdb_robust_mutex_pid == 0) {
@@ -814,17 +841,17 @@ _PUBLIC_ bool tdb_runtime_check_for_robust_mutexes(void)
 		ret = pthread_mutex_lock(m);
 		nwritten = write(pipe_up[1], &ret, sizeof(ret));
 		if (nwritten != sizeof(ret)) {
-			exit(1);
+			_exit(1);
 		}
 		if (ret != 0) {
-			exit(1);
+			_exit(1);
 		}
 		nread = read(pipe_down[0], &c, 1);
 		if (nread != 1) {
-			exit(1);
+			_exit(1);
 		}
 		/* leave locked */
-		exit(0);
+		_exit(0);
 	}
 	if (tdb_robust_mutex_pid == -1) {
 		goto cleanup_sig_child;
@@ -869,7 +896,7 @@ _PUBLIC_ bool tdb_runtime_check_for_robust_mutexes(void)
 			goto cleanup_child;
 		}
 	}
-	signal(SIGCHLD, tdb_robust_mutext_old_handler);
+	tdb_robust_mutex_setup_sigchild(tdb_robust_mutext_old_handler, NULL);
 
 	ret = pthread_mutex_trylock(m);
 	if (ret != EOWNERDEAD) {
@@ -915,7 +942,7 @@ cleanup_child:
 		}
 	}
 cleanup_sig_child:
-	signal(SIGCHLD, tdb_robust_mutext_old_handler);
+	tdb_robust_mutex_setup_sigchild(tdb_robust_mutext_old_handler, NULL);
 cleanup_m:
 	pthread_mutex_destroy(m);
 cleanup_ma:
